import sys
from functools import wraps


def Pipe(pipe_transformer, argument_transformer):
    def wrapper(func):
        @wraps(func)
        def decorator(*args, **kwargs):
            # If there is stdin input, get it.
            if not sys.stdin.isatty():
                arg = pipe_transformer.transform(sys.stdin.read())
            # Otherwise, it expects commandline arguments.
            else:
                try:
                    arg = argument_transformer.transform(sys.argv[1])
                except IndexError:
                    message = 'Please provide arguments or pipe it through the function.'
                    raise IndexError(message)
            return func(arg)
        return decorator
    return wrapper



from collections import OrderedDict

import mygene

from biopipe.contexts import no_output
from biopipe.decorators import Pipe
from biopipe.transformers import FileReadTransformer
from biopipe.transformers import ListTransformer

HOMO_SAPIENS_SPECIES_ID = 9606
mg = mygene.MyGeneInfo()


def get_first_item(items):
    """Return the first item of items. If 'items' is a single value, just return it."""
    return mygene.alwayslist(items)[0]


@Pipe(pipe_transformer=ListTransformer, argument_transformer=FileReadTransformer(ListTransformer))
def ensg2symbol(ensembl_ids=None):
    """Convert Ensembl gene ids into gene symbols."""
    # Silently query mygene server.
    with no_output():
        query_results = mg.getgenes(ensembl_ids, fields='symbol', species=HOMO_SAPIENS_SPECIES_ID)
    raw_result = [(query_result['query'], query_result['symbol']) for query_result in query_results if 'symbol' in query_result]

    best_result = OrderedDict()
    for query_id, symbol in raw_result:
        if query_id not in best_result:
            best_result[query_id] = symbol

    print('\n'.join(best_result.values()))


@Pipe(pipe_transformer=ListTransformer, argument_transformer=FileReadTransformer(ListTransformer))
def symbol2ensg(symbols=None):
    """Convert gene symbols into Ensembl gene ids."""
    # Silently query mygene server.
    with no_output():
        query_results = mg.querymany(symbols, scopes='symbol', fields='ensembl.gene', species=9606)
    raw_result = [(query_result['query'], get_first_item(query_result['ensembl'])['gene'])
                  for query_result in query_results
                  if 'ensembl' in query_result]

    best_result = OrderedDict()
    for query_symbol, ensembl_id in raw_result:
        if query_symbol not in best_result:
            best_result[query_symbol] = ensembl_id

    print('\n'.join(best_result.values()))



import os
import sys
from contextlib import contextmanager


@contextmanager
def no_output():
    old_stdout, sys.stdout = sys.stdout, open(os.devnull, 'w')
    try:
        yield None
    finally:
        sys.stdout = old_stdout



class BasicTransformer:
    def __init__(self):
        raise NotImplementedError

    @staticmethod
    def transform():
        raise NotImplementedError


class CompositeTransformer:
    def __init__(self, childTransformer=None):
        self.childTransformer = childTransformer

    def transform():
        raise NotImplementedError


class ListTransformer(BasicTransformer):
    def __init__(self):
        super(ListTransformer, self).__init__()

    @staticmethod
    def transform(string):
        return string.splitlines()


class FileReadTransformer(CompositeTransformer):
    def __init__(self, childTransformer=None):
        super(FileReadTransformer, self).__init__(childTransformer)

    def transform(self, file_path):
        with open(file_path) as inFile:
            string = inFile.read()
            if self.childTransformer is not None:
                return self.childTransformer.transform(string)
            else:
                return inFile.read()
